import os
import logging
import logging.config
from logging import Formatter
from tlib.base import TestHelper


def get_tlib_logger(request=None):
    """
    Returns logger named tlib.\n
    This logger should only be used by TLib modules. To log from a test or test module use get_test_logger\n
    Logger will get flushed at the end to ensure data is not lost if tests exits abnormally
    :param request
    """
    if request is None:
        return get_logger(name="tlib")
    else:
        return get_logger(path=request.fspath.strpath, name="tlib")


def get_adb_logger(request=None):
    """
    Returns logger named adb.\n
    This logger should only be used by adb modules. To log from a test or test module use get_test_logger\n
    Logger will get flushed at the end to ensure data is not lost if tests exits abnormally
    """
    if request is None:
        return get_logger(name="adb")
    else:
        return get_logger(path=request.fspath.strpath, name="adb")

def get_test_logger(request=None):
    """
    Returns logger named test.\n
    This logger should only be used by test modules. To log from a test or test module use get_test_logger\n
    Logger will get flushed at the end to ensure data is not lost if tests exits abnormally
    """
    if request is None:
        return get_logger(name="test")
    else:
        return get_logger(path=request.fspath.strpath, name="test")

def get_logger(path=None, name=None):
    """
    Returns a logger that has a console and a file handler.\n

    @param name: Logger to return
    @type name: str
    @return:
    """

    def find_log_folder(path):
        log_folder = None
        for i in range(1, 10):
            path = os.path.abspath(os.path.join(path, os.pardir))

            #Check if there is a folder 'logs'
            folder = os.path.join(path, 'logs')
            if os.path.exists(folder):
                return folder

        return None

    # find location of .log file
    #Go up to three levels to find log folder. Search starts in current directory
    if name is None:
        raise ValueError("Didn't provide a logger name")


    log_folder = None
    if path is None:
        log_folder = find_log_folder(os.path.dirname(__file__)) or find_log_folder(os.path.abspath(os.path.curdir))
        # path = os.path.dirname(__file__)
    else:
        log_folder = find_log_folder(path)

    if log_folder is None:
        raise IOError("Couldn't find folder 'logs'. Create it under your project folder and try again")

    #Create logger
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    #Add a file handler
    if not logger.handlers:
        fh = logging.FileHandler(os.path.join(log_folder, 'test.log'))
        logger.addHandler(fh)
        logger.propagate = 0
        fh.setLevel(logging.DEBUG)

        #Set log format
        formatter = Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)

    return logger